@@ -46,12 +46,12 @@ def type_validators(type_: SusType):
     if len(vals) == 0:
         return "{}"
     return "{ " + ', '.join(vals) + " }"
-def type_to_amogus(type_: SusType, obj_types: Dict[str, str]) -> str:
+def type_to_speedapi(type_: SusType, obj_types: Dict[str, str]) -> str:
     if type_.name == "List":
-        elements = type_to_amogus(type_.args[0], obj_types)
-        return f"new amogus.repr.List({elements}, {type_.args[1]}, {type_validators(type_)})"
+        elements = type_to_speedapi(type_.args[0], obj_types)
+        return f"new speedapi.repr.List({elements}, {type_.args[1]}, {type_validators(type_)})"
     if type_.name == "Entity":
-        return "new amogus.repr.Entity()"
+        return "new speedapi.repr.Entity()"
 
     if type_.name in obj_types:
         # determine the type of the object from its name
@@ -64,25 +64,25 @@ def type_to_amogus(type_: SusType, obj_types: Dict[str, str]) -> str:
         }[obj_types[type_.name]]
 
         if t_name == "Entity":
-            return "new amogus.repr.Entity()"
+            return "new speedapi.repr.Entity()"
 
         if t_name == "Compound":
-            return f"new amogus.repr.FieldArray({type_.name}Spec)"
+            return f"new speedapi.repr.FieldArray({type_.name}Spec)"
 
         if t_name == "EnumOrBf":
-            return f"new amogus.repr.EnumOrBf<{type_.name}>({type_.name}_SIZE)"
+            return f"new speedapi.repr.EnumOrBf<{type_.name}>({type_.name}_SIZE)"
         
-        return f"new amogus.repr.{t_name}({type_.name})"
+        return f"new speedapi.repr.{t_name}({type_.name})"
 
     name = type_.name
     if name == "Int" and type_.args[0] > 4:
         name = "BigInteger"
 
-    return f"new amogus.repr.{name}({', '.join([str(x) for x in type_.args] + [''])}{type_validators(type_)})"
+    return f"new speedapi.repr.{name}({', '.join([str(x) for x in type_.args] + [''])}{type_validators(type_)})"
 
 def write_output(root_file: File, target_dir: str) -> None:
     proj_name = path.splitext(path.basename(root_file.path))[0]
-    header = ("/* Generated by AMOGUS SUSC (https://github.com/amogus-api/susc)\n"
+    header = ("/* Generated by SpeedAPI SUSC (https://github.com/speedapi/susc)\n"
     f" * Project name: {proj_name}\n *\n"
     "" + LICENSE + "\n */\n\n")
 
@@ -99,7 +99,7 @@ def write_output(root_file: File, target_dir: str) -> None:
 
     with open(path.join(target_dir, "index.ts"), "w") as f:
         f.write(header)
-        f.write("import * as amogus from \"amogus-driver\";\n\n")
+        f.write("import * as speedapi from \"@speedapi/driver\";\n\n")
 
         # write compounds
         compounds = [t for t in root_file.things if isinstance(t, SusCompound)]
@@ -136,7 +136,7 @@ def write_output(root_file: File, target_dir: str) -> None:
             f.write("};\n")
             # write class
             write_docstr(f, conf)
-            f.write(f"export class {name} extends amogus.Confirmation<typeof {name}Spec> {'{'}\n")
+            f.write(f"export class {name} extends speedapi.Confirmation<typeof {name}Spec> {'{'}\n")
             f.write("\tconstructor() {\n")
             f.write(f"\t\tsuper({name}Spec, {conf.value});\n")
             f.write("\t}\n")
@@ -162,7 +162,7 @@ def write_output(root_file: File, target_dir: str) -> None:
             f.write("};\n")
             # write class
             write_docstr(f, method)
-            f.write(f"export class {name} extends amogus.Method<typeof {name}Spec> {'{'}\n")
+            f.write(f"export class {name} extends speedapi.Method<typeof {name}Spec> {'{'}\n")
             f.write("\tconstructor() {\n")
             f.write(f"\t\tsuper({name}Spec, {method.value}, undefined);\n")
             f.write("\t}\n")
@@ -170,11 +170,11 @@ def write_output(root_file: File, target_dir: str) -> None:
             # write function
             write_docstr(f, method)
             f.write(f"async function {snake_to_camel(method.name)}(\n")
-            f.write("\tthis: any | amogus.BoundSession,\n")
-            f.write(f"\tparams: amogus.repr.FieldValue<typeof {name}Spec[\"params\"]>,\n")
-            f.write(f"\tconfirm?: amogus.ConfCallback<{name}>,\n")
-            f.write("\tsession?: amogus.Session<amogus.SpecSpace>\n")
-            f.write(f"): Promise<amogus.repr.FieldValue<typeof {name}Spec[\"returns\"]>> {'{'}\n")
+            f.write("\tthis: any | speedapi.BoundSession,\n")
+            f.write(f"\tparams: speedapi.repr.FieldValue<typeof {name}Spec[\"params\"]>,\n")
+            f.write(f"\tconfirm?: speedapi.ConfCallback<{name}>,\n")
+            f.write("\tsession?: speedapi.Session<speedapi.SpecSpace>\n")
+            f.write(f"): Promise<speedapi.repr.FieldValue<typeof {name}Spec[\"returns\"]>> {'{'}\n")
             f.write(f"\tconst method = new {name}();\n")
             f.write(f"\tmethod.params = params;\n")
             f.write(f"\treturn await (session ?? this.$session).invokeMethod(method, confirm);\n")
@@ -183,6 +183,8 @@ def write_output(root_file: File, target_dir: str) -> None:
         # write entities
         entities = [t for t in root_file.things if isinstance(t, SusEntity)]
         for entity in entities:
+            id_field = [f for f in entity.fields if f.name == "id"][0]
+
             # write method specs and classes
             for method in entity.methods:
                 name = f"{entity.name}_{snake_to_pascal(method.name)}"
@@ -197,10 +199,12 @@ def write_output(root_file: File, target_dir: str) -> None:
                 write_field_array(f, method.returns, objs)
                 f.write("\t},\n")
                 conf_names = ", ".join(f"new {snake_to_pascal(conf)}()" for conf in method.confirmations)
-                f.write(f"\tconfirmations: [{conf_names}]\n")
+                f.write(f"\tconfirmations: [{conf_names}],\n")
+                if not method.static:
+                    f.write(f"\tentityIdRepr: {type_to_speedapi(id_field.type_, objs)}\n")
                 f.write("};\n")
                 write_docstr(f, method)
-                f.write(f"export class {name} extends amogus.Method<typeof {name}Spec> {'{'}\n")
+                f.write(f"export class {name} extends speedapi.Method<typeof {name}Spec> {'{'}\n")
                 f.write("\tconstructor() {\n")
                 f.write(f"\t\tsuper({name}Spec, {method.value + (128 if method.static else 0)}, {entity.value});\n")
                 f.write("\t}\n")
@@ -220,10 +224,10 @@ def write_output(root_file: File, target_dir: str) -> None:
             f.write("};\n")
             # write class
             write_docstr(f, entity)
-            f.write(f"export class {name} extends amogus.Entity<typeof {name}Spec> {'{'}\n")
-            f.write("\tprotected static readonly session?: amogus.Session<amogus.SpecSpace>;\n")
-            f.write("\treadonly dynSession?: amogus.Session<amogus.SpecSpace>;\n\n")
-            f.write(f"\tconstructor(value?: amogus.repr.FieldValue<typeof {name}Spec[\"fields\"]>) {'{'}\n")
+            f.write(f"export class {name} extends speedapi.Entity<typeof {name}Spec> {'{'}\n")
+            f.write("\tprotected static readonly session?: speedapi.Session<speedapi.SpecSpace>;\n")
+            f.write("\treadonly dynSession?: speedapi.Session<speedapi.SpecSpace>;\n\n")
+            f.write(f"\tconstructor(value?: speedapi.repr.FieldValue<typeof {name}Spec[\"fields\"]>) {'{'}\n")
             f.write(f"\t\tsuper({name}Spec, {entity.value}, value);\n")
             f.write("\t}\n")
 
@@ -240,10 +244,10 @@ def write_output(root_file: File, target_dir: str) -> None:
                 static = "static " if method.static else ""
                 protected = "protected " if method.name == "get" else ""
                 f.write(f"\t{protected}{static}async {snake_to_camel(method.name)}(\n")
-                f.write(f"\t\tparams: amogus.repr.FieldValue<typeof {name}Spec[\"params\"]>,\n")
-                f.write(f"\t\tconfirm?: amogus.ConfCallback<{name}>,\n")
-                f.write("\t\tsession?: amogus.Session<amogus.SpecSpace>\n")
-                f.write(f"\t): Promise<amogus.repr.FieldValue<typeof {name}Spec[\"returns\"]>> {'{'}\n")
+                f.write(f"\t\tparams: speedapi.repr.FieldValue<typeof {name}Spec[\"params\"]>,\n")
+                f.write(f"\t\tconfirm?: speedapi.ConfCallback<{name}>,\n")
+                f.write("\t\tsession?: speedapi.Session<speedapi.SpecSpace>\n")
+                f.write(f"\t): Promise<speedapi.repr.FieldValue<typeof {name}Spec[\"returns\"]>> {'{'}\n")
                 f.write(f"\t\tconst method = new {name}();\n")
                 f.write(f"\t\tmethod.params = params;\n")
                 if method.static:
@@ -255,14 +259,14 @@ def write_output(root_file: File, target_dir: str) -> None:
                 f.write("\t}\n")
 
             # write $get()
-            f.write("\n\tstatic async $get(id: bigint) {\n")
-            f.write(f"\t\treturn (await this.get({'{'} id {'}'})).entity as amogus.ValuedEntity<{entity.name}>;\n")
+            f.write(f"\n\tstatic async $get(id: speedapi.repr.TsType<typeof {entity.name}Spec[\"fields\"][\"required\"][\"id\"]>) {'{'}\n")
+            f.write(f"\t\treturn (await this.get({'{'} id {'}'})).entity as speedapi.ValuedEntity<{entity.name}>;\n")
             f.write("\t}\n")
 
             f.write("}\n\n\n")
 
         # write spec space
-        f.write("\nexport function $specSpace(session: amogus.Session<amogus.SpecSpace>) {\n")
+        f.write("\nexport function $specSpace(session: speedapi.Session<speedapi.SpecSpace>) {\n")
         f.write("\treturn {\n")
         f.write("\t\tspecVersion: \"2\" as const,\n")
         f.write(f"\t\tproject: \"{proj_id}\" as const,\n")
@@ -285,7 +289,7 @@ def write_output(root_file: File, target_dir: str) -> None:
         f.write("}\n\n\n")
 
         # write $bind()
-        f.write("\nexport function $bind(session: amogus.Session<amogus.SpecSpace>) {\n")
+        f.write("\nexport function $bind(session: speedapi.Session<speedapi.SpecSpace>) {\n")
         f.write(f"\tif(session.specSpace.project !== \"{proj_id}\")\n")
         f.write("\t\tthrow new Error(\"failed to $bind: project identifier does not match\")\n\n")
         f.write("\treturn {\n")
@@ -313,13 +317,13 @@ def write_field_array(f, fields, objs, indent=2):
     f.write(f"{indent}required: {'{'}\n")
     for field in [f for f in fields if f.optional is None]:
         write_docstr(f, field, len(indent) + 1)
-        f.write(f"{indent}\t{field.name}: {type_to_amogus(field.type_, objs)},\n")
+        f.write(f"{indent}\t{field.name}: {type_to_speedapi(field.type_, objs)},\n")
     f.write(f"{indent}{'}'},\n")
 
     f.write(f"{indent}optional: {'{'}\n")
     for field in [f for f in fields if f.optional is not None]:
         write_docstr(f, field, len(indent) + 1)
-        type_ = type_to_amogus(field.type_, objs)
+        type_ = type_to_speedapi(field.type_, objs)
         repr_class = type_[:type_.find("(")]
         f.write(f"{indent}\t{field.name}: [{field.optional}, {type_}] as const,\n")
     f.write(f"{indent}{'}'}\n")
